% Nonlinear CSTR  model

% 7 states
% 3 inputs
% 3 disturbances

% Initial values:
% X_Glu = 0; % g/L
% Glu = 20; % g/L
% Man = 20; % g/L
% Xyl = 20; % g/L
% P = 0; % M
% V = 0.7 % L
% V_RiboGlu = 1 % L
%
% Constraints
% % For x = [X_RiboGlu Glu, Man, Xyl, P, V, V_RiboGlu]
% x_lower = [0 0 0 0 0 0.7 0.7 ];
% script not to run
% x_upper = [ inf,  inf, inf, inf, inf 1 1];
%
% % For u = [F_RiboGlu_in , F_SSL_in, F_out] % 
% u_lower = [0 0 0 ]; 
% u_upper = [1 1 1]; 
%
function ODEsys = ODEmodel()
%addpath ('/users/dinark/Downloads/casadi-osx-matlabR2015a-v3.5.1/')
import casadi.*
ODEsys = struct; %Empty struct

% States, declaring them symbolically
X_RiboGlu = SX.sym('X_RiboGlu'); %Biomass, g/L
Glu = SX.sym('Glu'); %Substrate, g/L
Man = SX.sym('Man'); %Substrate, g/L
Xyl = SX.sym('Xyl'); %Substrate, g/L
P= SX.sym('P'); % Riboflavin, M
V=SX.sym('V'); % Reactor volume, L
V_RiboGlu=SX.sym('V_RiboGlu'); %RiboGlu reactor volume, L
%Filling up the ODEsys struct
ODEsys.x = [X_RiboGlu, Glu, Man, Xyl, P, V, V_RiboGlu]';
ODEsys.x_names = ['X_RiboGlu', 'Glu', 'Man', 'Xyl', 'P', 'V', 'V_RiboGlu']';

% Inputs - Normalized by reactor volume.
F_RiboGlu_in=SX.sym('F_RiboGlu_in'); %L/h
F_SSL_in=SX.sym('F_SSL_in'); %L/h - sugar stream
F_out= SX.sym('F_out');%L/h

ODEsys.u = [F_RiboGlu_in, F_SSL_in, F_out]';
ODEsys.u_names = [F_RiboGlu_in, F_SSL_in, F_out]';

% Uncertainties
% SSL feed in stream concentrations
SSL_Glu_in = SX.sym('SSL_Glu_in'); % g/L
SSL_Man_in = SX.sym('SSL_Man_in'); %g/L
SSL_Xyl_in = SX.sym('SSL_Xyl_in'); % g/L
ODEsys.d = [SSL_Glu_in,SSL_Man_in,SSL_Xyl_in]';
ODEsys.d0 = [36,110,45]'; % g/L  SSL
%% SyntheticSSL
%ODEsys.d0 = [20,20,20]'; % g/L  SSL

%Substrate concentrations
%Assuming the S_in are zero, as they come from a completed batch with
%sugars consumed.
%Sugars from RiboGlu strain
RiboGlu_Glu_in=0;
RiboGlu_Man_in=0;
RiboGlu_Xyl_in=0;
%Biomass concentrations
%RiboGlu
RiboGlu_X_RiboGlu_in=7;
%Riboflavin concentrations 
%Interested in the riboflavin produced in main reactor
RiboGlu_P_in=0;
SSL_P_in=0;


%% #####################################################
%                       Parameters
%              Obtained from Parameter_Esitmation_Strain

%% RiboGlu
mumax1_RiboGlu=0.1290;
mumax2_RiboGlu=0.7299;
mumax3_RiboGlu=0.0013;
Ks1_RiboGlu=0.0523;
Ks2_RiboGlu=523.9684;
Ks3_RiboGlu=834.3772;
Y_xs1_RiboGlu=-0.3806;
Y_xs2_RiboGlu=-0.1716;
Y_xs3_RiboGlu=-0.0050;
Y_px_RiboGlu=0.7663;
% #######################################
%               Constants
% #######################################

mu1_RiboGlu= mumax1_RiboGlu*Glu/(Ks1_RiboGlu+Glu);
mu2_RiboGlu = mumax2_RiboGlu*Man/(Ks2_RiboGlu+Man);
mu3_RiboGlu = mumax3_RiboGlu*Xyl/(Ks3_RiboGlu+Xyl);
rx1_RiboGlu = mu1_RiboGlu*X_RiboGlu;
rx2_RiboGlu = mu2_RiboGlu*X_RiboGlu;
rx3_RiboGlu = mu3_RiboGlu*X_RiboGlu;
rx_RiboGlu = rx1_RiboGlu + rx2_RiboGlu + rx3_RiboGlu;
rs1_RiboGlu = rx1_RiboGlu/Y_xs1_RiboGlu;
rs2_RiboGlu = rx2_RiboGlu/Y_xs2_RiboGlu;
rs3_RiboGlu = rx3_RiboGlu/Y_xs3_RiboGlu;
rp_RiboGlu=Y_px_RiboGlu*rx_RiboGlu;

%% Initial values of states

% Initial Values of states
ODEsys.x0 = [0, 20, 20, 20, 0, 0.7, 1]';
ODEsys.u0 = [0,0,0]';

% For x = [X_RiboGlu,  Glu, Man, Xyl, P, V, V_RiboGlu]
ODEsys.x_lower = [0, 0,0,0,0 0.7, 0.7]'; %
ODEsys.x_upper = [ inf, inf, inf, inf, inf, 1, 1]';

% For u = [F_RiboGlu_in, F_SSL_in, F_out] % 
  ODEsys.u_lower = [0, 0,0]'; 
  ODEsys.u_upper = [1, 1, 1]';
   %% Constant F_SSL_in of 0.1 L/h
%  ODEsys.u_lower = [0, 0.1,0]';
%  ODEsys.u_upper = [1, 0.1, 1]';

ODEsys.delta_U_lower = [ -inf, -inf, -inf]';
ODEsys.delta_U_upper = [ inf, inf, inf]';

%% Renaming Flow - expressions
D_Glu= F_SSL_in*(SSL_Glu_in-Glu)/V + F_RiboGlu_in*(RiboGlu_Glu_in-Glu)/V; 
D_Man=F_SSL_in*(SSL_Man_in-Man)/V + F_RiboGlu_in*(RiboGlu_Man_in-Man)/V;
D_Xyl=F_SSL_in*(SSL_Xyl_in-Xyl)/V + F_RiboGlu_in*(RiboGlu_Xyl_in-Xyl)/V;
%Biomass
%% Dilutions - change of concentrations because of streams in and out
D_RiboGlu= F_SSL_in*(0-X_RiboGlu)/V+ ...
    F_RiboGlu_in*(RiboGlu_X_RiboGlu_in-X_RiboGlu)/V;

D_P= F_SSL_in*(SSL_P_in-P)/V+ ...
     F_RiboGlu_in*(RiboGlu_P_in-P)/V ;
%%  ODEs
X_RiboGlu_dot = rx_RiboGlu + D_RiboGlu; 
Glu_dot = rs1_RiboGlu + D_Glu;
Man_dot = rs2_RiboGlu +  D_Man;
Xyl_dot= rs3_RiboGlu +  D_Xyl;
P_dot = rp_RiboGlu + D_P;
V_dot=F_RiboGlu_in +  F_SSL_in - F_out; 
V_RiboGlu_dot=-F_RiboGlu_in;
ODEsys.x_dot = [X_RiboGlu_dot, Glu_dot, Man_dot, Xyl_dot, P_dot, V_dot, V_RiboGlu_dot]'; % endre

end
