%% RiboXyl strain with glucose, xylose and SSL sugars.
function results =ParameterEstimation_RiboXyl_Multiple() 
close all
% Estimation of ODE parameters by non-linear least squares optimization 
% solver. 
% -------------------------------------------------------------
%% Initial guesses of the parameters  
initParams=[0.0675    0.0003    0.1082    0.0905  653.7131    2.0783   -0.2396   -0.0001   -0.2876    0.9751];% estimating 10 parameters

%exper=[0 0]; tspan=0;
%% Known values
%correction factor of the lab
cfac=0.3634;
%molarmasses g/mmol
M_glu=0.18;
M_man=0.18;
M_xyl=0.15013;

%% Defining x0 and timespan
x0 = 0;
tspan = 0;
%%        Glucose
%###########################
t_end = 60;%Time at the end of the batch phase
rawdata1 = readtable('RiboXyl_Glu.xlsx');

for i = 1:width(rawdata1)
    if iscell(rawdata1{:,i})
        data(:,i) = str2double(rawdata1{:,i});
    else
        data(:,i) = rawdata1{:,i};
    end
end

% Finding slopes for substrate and product
Glu_peak = data(2,7:10);
Glu_conc = data(1,7:10);
Glu_slope=polyfit(Glu_conc,Glu_peak,1);
Glu_slope=Glu_slope(1);

Ribo_peak = data(2,11:13);
Ribo_conc = data(1,11:13);
Ribo_slope=polyfit(Ribo_conc,Ribo_peak,1);
Ribo_slope=Ribo_slope(1);

% The first values of each state
time_data = data(:,1); % Time column in data sheet
timespan{1} = time_data(time_data<t_end)';%Including times up to t_end
X_exp = data(1:length(timespan{1}),2)*cfac; %concentration [g/L]
Glu_exp = (data(1:length(timespan{1}),3)/Glu_slope*10)*M_glu; %concentration [g/L]
Man_exp= zeros(size(Glu_exp));%concentration [g/L]
Xyl_exp= zeros(size(Glu_exp));%concentration [g/L]
P_exp = (data(1:length(timespan{1}),4)/Ribo_slope*10)*1000; %concentration [M]
x_init(1,:) = [X_exp(1),Glu_exp(2), Man_exp(1), Xyl_exp(1), P_exp(1)];%initial states
exper{1} = [X_exp,Glu_exp, Man_exp, Xyl_exp, P_exp];%experimental
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');


%%      Xylose
%###########################
t_end2=40; %Time at the end of the batch phase
rawdata2 = readtable('RiboXyl_Xyl.xlsx');
for i = 1:width(rawdata2)
    if iscell(rawdata2{:,i})
        data2(:,i) = str2double(rawdata2{:,i});
    else
        data2(:,i) = rawdata2{:,i};
    end
end
% Finding slopes for substrate and product
Xyl_peak = data2(2,7:10);
Xyl_conc = data2(1,7:10);
Xyl_slope = polyfit(Xyl_conc,Xyl_peak,1);
Xyl_slope = Xyl_slope(1);

Ribo_peak2 = data2(2,11:13);
Ribo_conc2 = data2(1,11:13);
Ribo_slope2=polyfit(Ribo_conc2,Ribo_peak2,1);
Ribo_slope2=Ribo_slope2(1);

% The first values of each state
time_data = data2(:,1); % Time column in data sheet
timespan{2} = time_data(time_data<t_end2)';%Including times up to t_end2
X_exp = data2(1:length(timespan{2}),2)*cfac; %concentration [g/L]
Xyl_exp = (data2(1:length(timespan{2}),3)/Xyl_slope*10)*M_xyl; %concentration [g/L]
Glu_exp = zeros(size(Xyl_exp));%concentration [g/L]
Man_exp = zeros(size(Xyl_exp));%concentration [g/L]
P_exp = (data2(1:length(timespan{2}),4)/Ribo_slope2*10)*1000; %concentration [M]
x_init(2,:) = [X_exp(1),Glu_exp(1), Man_exp(1),Xyl_exp(1),P_exp(1)]; %initial states
exper{2} = [X_exp,Glu_exp, Man_exp, Xyl_exp, P_exp]; %experimental
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');



%%      SSL
%###########################
t_end3 = 66;%Time at the end of the batch phase

rawdata3 = readtable('RiboXyl_SSL.xlsx');
for i = 1:width(rawdata3)
    if iscell(rawdata3{:,i})
        data3(:,i) = str2double(rawdata3{:,i});
    else
        data3(:,i) = rawdata3{:,i};
    end
end

% Finding slopes for S and P
Glu_peak3 = data3(2,7:10);
Glu_conc3 = data3(1,7:10);
Glu_slope2=polyfit(Glu_conc3,Glu_peak3,1);
Glu_slope2=Glu_slope2(1);

Man_peak2 = data3(3,[7,9,10]);
Man_conc2 = data3(1,[7,9,10]);
Man_slope2=polyfit(Man_conc2,Man_peak2,1);
Man_slope2=Man_slope2(1);

Xyl_peak = data3(4,7:10);
Xyl_conc = data3(1,7:10);
Xyl_slope=polyfit(Xyl_conc,Xyl_peak,1);
Xyl_slope=Xyl_slope(1);

Ribo_peak3 = data3(2,12:14);
Ribo_conc3 = data3(1,12:14);
Ribo_slope3=polyfit(Ribo_conc3,Ribo_peak3,1);
Ribo_slope3=Ribo_slope3(1);

% The first values of each state
time_data = data3(:,1); % Time column in data sheet
timespan{3} = time_data(time_data<t_end3)';%Including times up to t_end3
X_exp = data3(1:length(timespan{3}),2)*cfac; %concentration [g/L]
Glu_exp = (data3(1:length(timespan{3}),3)/Glu_slope2*10)*M_glu;%concentration [g/L]
Man_exp = (data3(1:length(timespan{3}),4)/Man_slope2*10)*M_man; %concentration [g/L]
Xyl_exp= (data3(1:length(timespan{3}),5)/Xyl_slope*10)*M_xyl;%concentration [g/L]
P_exp = (data3(1:length(timespan{3}),6)/Ribo_slope3*10)*1000; %concentration [M]
x_init(3,:) = [X_exp(1),Glu_exp(1), Man_exp(1),Xyl_exp(1), P_exp(1)]; %initial states
exper{3} = [X_exp,Glu_exp, Man_exp, Xyl_exp, P_exp]; %experimental
opts = optimset('TolFun', 1e-12, 'TolX', 1e-12, 'MaxIter', 1000, 'Diagnostics', 'off', 'Display', 'iter');

%% Defining lower and upper bounds for each parameter

loBound = [0.0 0.0 0.0 0.0 0.0 0.0 -1 -1 -1 0.0];
upBound = [5 5 5 1000 1000 1000 0 0 0 1];  

res = lsqnonlin(@residual,  initParams, loBound, upBound, opts);


% Plotting
%Glucose plot
b=figure();
plot(timespan{1},Y{1}(:,[1, 2, 5]), 'LineWidth',1.5)
hold on
plot(timespan{1},exper{1}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{1},exper{1}(:,2),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{1},exper{1}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]','Glu [g/L]','P [M]','X_{exp} [g/L]','Glu_{exp} [g/L]','P_{exp} [M]')
%title('RiboXyl strain with glucose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 25])
editFigureProperties(b)
saveas(gcf,'RXG.png')


%Xylose plot
c=figure();
plot(timespan{2},Y{2}(:,[1, 4, 5]), 'LineWidth',1.5)
hold on
plot(timespan{2},exper{2}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{2},exper{2}(:,4),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{2},exper{2}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]','Xyl [g/L]','P [M]','X_{exp} [g/L]','Xyl_{exp} [g/L]','P_{exp} [M]')
%title('RiboXyl strain with xylose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 25])
editFigureProperties(c)
saveas(gcf,'RXX.png')


%SSL plot
d=figure();
subplot(3,1,1);
plot(timespan{3},Y{3}(:,[1, 2, 5]), 'LineWidth',1.5)%(:,[1, 2, 5])
hold on
plot(timespan{3},exper{3}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,2),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]','Glu [g/L]','P [M]','X_{exp} [g/L]','Glu_{exp} [g/L]','P_{exp} [M]')
legend('Location','northeastoutside')
%title('RiboXyl strain with synthetic SSL as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 30])

subplot(3,1,2)
plot(timespan{3},Y{3}(:,[1, 3, 5]), 'LineWidth',1.5)
hold on
plot(timespan{3},exper{3}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,3),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]','Man [g/L]' ,'P [M]','X_{exp} [g/L]','Man_{exp} [g/L]','P_{exp} [M]')
legend('Location','northeastoutside')
%title('RiboXyl strain with mannose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 30])

subplot(3,1,3)
plot(timespan{3},Y{3}(:,[1, 4, 5]), 'LineWidth',1.5)
hold on
plot(timespan{3},exper{3}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,4),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]', 'Xyl [g/L]' ,'P [M]','X_{exp} [g/L]','Xyl_{exp} [g/L]','P_{exp} [M]')
legend('Location','northeastoutside')
%title('RiboXyl strain with xylose as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 30])
editFigureProperties(d)
saveas(gcf,'RXSSL.png')

e=figure();
plot(timespan{3},Y{3}, 'LineWidth',1.5)
hold on
plot(timespan{3},exper{3}(:,1),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,2),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,3),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,4),'o','MarkerSize',10, 'LineWidth',3)
plot(timespan{3},exper{3}(:,5),'o','MarkerSize',10, 'LineWidth',3)
legend('X [g/L]','Glu [g/L]', 'Man [g/L]', 'Xyl [g/L]' ,'P [M]','X_{exp} [g/L]','Glu_{exp} [g/L]', 'Man_{exp} [g/L]', 'Xyl_{exp} [g/L]','P_{exp} [M]')
legend('Location','northeastoutside')
%title('RiboXyl strain with synthetic SSL as carbon source')
xlabel('Time [h]')
ylabel('Concentration')
ylim([0 30])
editFigureProperties(e)
saveas(gcf,'RXSSL_2.png')

%initParams=b at first, but changes through the loop
   function R = residual(b)
        
       a = b;
       
       R = [];
       Y = [];
       
       for j = 1:3
           tspan = timespan{j};
           x0 = x_init(j,:);
       [T ,Y{j} ]= modelsolve (a);
        R = [R; exper{j} - Y{j}];
       end
       
       
      %residual = exper(:,2) - Y(:,2);
     
       %R = residual(:);
       R(isnan(R)) = 0;
       R(:,1) = 10*R(:,1); %Biomass is weighted more
       
       results=a;     

   end

function [ T, Y ] = modelsolve( a)
%% All the parameters that will be estimated are defined as a(i):
mumax1=a(1);
mumax2= a(2);
mumax3= a(3);
Ks1=a(4);
Ks2=a(5);
Ks3=a(6);
Y_xs1=a(7);
Y_xs2=a(8);
Y_xs3=a(9);
Y_px=a(10);

%% Solve ODE

[T,Y] = ode45(@model, tspan, x0, []);

function x_dot = model(t, x)

% Unpack initial values    

X=x(1);
Glu=x(2);
Man=x(3);
Xyl=x(4);
P=x(5);
%% Define all known/fixed parameters
mu1= mumax1*Glu/(Ks1+Glu);
mu2 = mumax2*Man/(Ks2+Man);
mu3 = mumax3*Xyl/(Ks3+Xyl);
rx1 = mu1*X;
rx2 = mu2*X;
rx3 = mu3*X;
rx = rx1 + rx2 + rx3;
rs1 = rx1/Y_xs1;
rs2 = rx2/Y_xs2;
rs3 = rx3/Y_xs3;
rp=Y_px*rx;

%Differential equations
X_dot = rx;
Glu_dot = rs1;
Man_dot = rs2;
Xyl_dot= rs3;
P_dot = rp;

x_dot = [X_dot, Glu_dot, Man_dot, Xyl_dot, P_dot]';

    
end

end

end


